<!DOCTYPE gui SYSTEM "kpartgui.dtd">
<gui name="kalzium" version="5">
<MenuBar>
  <Menu name="file_menu"><text>&amp;File</text>
    <Action name="saveAs"/>  
    <Action name="file_exporter" /> 
    <Action name="tools_obconverter" />
    <Separator />     
    <Action name="quit"/>
  </Menu>
  <Menu name="table_menu"><text>&amp;View</text>
    <Action name="view_table" /> 
    <Action name="view_numerationtype" />
    <Action name="view_look_onlyschemes" />
    <Action name="view_look_onlygradients" />
    <Separator />
    <Action name="view_legend" />
    <Action name="view_sidebar"/>
    <Action name="view_tablebar"/>
  </Menu>
  <Menu name="miscmenu"><text>&amp;Tools</text>
    <Action name="tools_moleculeviewer" /> 
    <Action name="tools_isotopetable" />  
    <Action name="tools_plotdata" />
    <Action name="tools_calculate" />
    <Action name="tools_eqsolver" />
    <Separator />
    <Action name="tools_rs" />      
    <Action name="tools_glossary" />    
    <Action name="tools_tables" />
  </Menu>
</MenuBar>

 <ToolBar noMerge="1" name="mainToolBar" alreadyVisited="1">
  <text>Main Toolbar</text>
  <Action name="view_table"/>
  <Action name="view_look_onlyschemes"/>
  <Action name="view_look_onlygradients"/>  
  <Action name="tools_isotopetable"/>
  <Action name="tools_moleculeviewer"/>
  <Action name="tools_calculate" />  
  <Action name="tools_plotdata" />
 </ToolBar>
</gui>
